#:coding=utf-8:

DEFAULT_USER_BACKENDS={
    'default': {
        'backend': 'newauth.backend.BasicUserBackend',
        'user': 'newauth.api.BasicUserBase', # TODO: Sane default?
        'anon_user': 'newauth.api.BasicAnonymousUser', # TODO: Sane default?
    }
}

DEFAULT_SESSION_KEY = '_newauth_user'
DEFAULT_USER_PROPERTY='auth_user'
DEFAULT_PASSWORD_ALGO='sha1'
REDIRECT_FIELD_NAME = 'next'

# Base logger name used for logging
LOGGER_NAME='newauth'



#:coding=utf-8:

from django.core.exceptions import ImproperlyConfigured
from django.utils.lru_cache import lru_cache

from newauth.api import _get_backend_data

__all__ = (
    'User',
    'AnonymousUser',
    'get_user_model',
    'get_anonymous_user_model',
)


@lru_cache()
def _get_user_models(model_name=None):
    from newauth.api import import_string

    if model_name is None:
        model_name = 'default'

    backend_data = _get_backend_data()

    try:
        model_path = backend_data[model_name]['user']
        anon_model_path = backend_data[model_name]['anon_user']
    except IndexError:
        if model_name is 'default':
            raise ImproperlyConfigured('A "default" user model is not specified. You must specify a "default" user model. Or maybe NEWAUTH_USER_MODELS isn\'t a correctly defined dict?')
        else:
            raise ImproperlyConfigured('Error importing User model class with name "%s". Is NEWAUTH_USER_MODELS a correctly defined dict?' % model_name)

    try:
        UserCls = import_string(model_path)
    except ImportError:
        raise ImproperlyConfigured('Error importing user model class: "%s"' % model_path)
    except AttributeError:
        raise ImproperlyConfigured('Module does not define a class "%s"' % model_path)

    try:
        AnonUserCls = import_string(anon_model_path)
    except ImportError:
        raise ImproperlyConfigured('Error importing Anonymous user model class: "%s"' % anon_model_path)
    except AttributeError:
        raise ImproperlyConfigured('Module does not define a class "%s"' % anon_model_path)
    return UserCls, AnonUserCls


def get_user_model(model_name=None):
    """
    Used to get access to defined user models.

    from newauth.models import User, get_user_model

    OtherUser = get_user_model('other')

    class MyProfile(models.Model):
        user = models.ForeignKey(User)
        other_user_type = models.ForeignKey(OtherUser)
    """
    return _get_user_models(model_name)[0]

def get_anonymous_user_model(model_name=None):
    return _get_user_models(model_name)[1]

try:
    User = get_user_model('default')
    AnonymousUser = get_anonymous_user_model('default')
except (IndexError, KeyError), e:
    raise ImproperlyConfigured('A "default" user model is not specified. You must specify a "default" user model. Or maybe NEWAUTH_USER_MODELS isn\'t a correctly defined dict?')



from importlib import import_module


class AuthTestCaseMixin(object):
    """
    from django.test import TestCase

    class MyTestCase(TestCase, AuthTestCaseMixin):
        def test_xxx(self):
            self.auth_login(username='spam', password='egg')
            # write some tests
    """

    def auth_login(self, **credentials):
        """
        Sets the Client to appear as if it has successfully logged into a site.

        Returns True if login is possible; False if the provided credentials
        are incorrect, or the user is inactive, or if the sessions framework is
        not available.
        """
        from django.http import HttpRequest
        from django.conf import settings
        from django.contrib.sessions.middleware import SessionMiddleware
        from newauth.middleware import AuthMiddleware
        from newauth.api import authenticate, login
        user = authenticate(**credentials)
        if user and 'django.contrib.sessions' in settings.INSTALLED_APPS:
            engine = import_module(settings.SESSION_ENGINE)

            # Create a fake request to store login details.
            request = HttpRequest()
            SessionMiddleware().process_request(request)
            AuthMiddleware().process_request(request)
            if self.client.session:
                request.session = self.client.session
            else:
                request.session = engine.SessionStore()
            login(request, user)

            # Save the session values.
            request.session.save()

            # Set the cookie to represent the session.
            session_cookie = settings.SESSION_COOKIE_NAME
            self.client.cookies[session_cookie] = request.session.session_key
            cookie_data = {
                'max-age': None,
                'path': '/',
                'domain': settings.SESSION_COOKIE_DOMAIN,
                'secure': settings.SESSION_COOKIE_SECURE or None,
                'expires': None,
            }
            self.client.cookies[session_cookie].update(cookie_data)

            return True
        return False




#:coding=utf-8:

from django import VERSION as DJANGO_VERSION
from django.conf import settings

if DJANGO_VERSION >= (1, 10):
    from django.utils.deprecation import MiddlewareMixin
else:
    MiddlewareMixin = object

from newauth.constants import DEFAULT_USER_PROPERTY


class LazyUser(object):
    """
    Lazily creates the user object.
    """
    def __get__(self, request, obj_type=None):
        if not hasattr(request, '_newauth_cached_user'):
            from newauth.api import get_user_from_request
            request._newauth_cached_user = get_user_from_request(request)
        return request._newauth_cached_user


class AuthMiddleware(MiddlewareMixin):
    """
    Middleware for getting the current user object
    and attaching it to the request.
    """
    def process_request(self, request):
        user_prop = getattr(settings, 'NEWAUTH_USER_PROPERTY', DEFAULT_USER_PROPERTY)
        setattr(request.__class__, user_prop, LazyUser())
        return None



#:coding=utf-8:

from django.utils.translation import ugettext_lazy as _
from django import forms

from newauth.api import authenticate

__all__ = (
    'BaseAuthForm',
    'BasicAuthForm',
)

class BaseAuthForm(forms.Form):
    """
    A base authentication form. Authentication forms
    can subclass this form and implement the
    get_credentials() method which will return a
    dictionary of credentials from self.cleaned_data
    that can be used with authenticate()
    """
    default_error_messages = {
        'auth_failure': _("Please enter a correct credentials."),
    }

    def __init__(self, *args, **kwargs):
        super(BaseAuthForm, self).__init__(*args, **kwargs)
        self.user_cache = None

    def get_credentials(self):
        """
        Gets credentials as a dict object from self.cleaned_data
        """
        raise NotImplemented

    def clean(self):
        credentials = self.get_credentials()
        if credentials:
            self.user_cache = authenticate(**credentials)
            if self.user_cache is None:
                raise forms.ValidationError(
                    self.default_error_messages['auth_failure'],
                    code='auth_failure',
                    params={'username': _("username")},
                )
        return self.cleaned_data

    def get_user_id(self):
        if self.user_cache:
            return self.user_cache.pk
        return None

    def get_user(self):
        """
        Gets the cached user object for the user that logged in using
        this form.
        """
        return self.user_cache

class BasicAuthForm(BaseAuthForm):
    """
    A basic authentication form that will authenticate
    a user using a username and password. Useful for
    subclasses of BasicUser.
    """
    default_error_messages = {
        'auth_failure': _("Please enter a correct %(username)s and password. Note that both fields may be case-sensitive."),
    }

    username = forms.CharField(label=_("username"), max_length=30)
    password = forms.CharField(label=_("password"), widget=forms.PasswordInput)

    def get_credentials(self):
        return {
            'username': self.cleaned_data.get('username'),
            'password': self.cleaned_data.get('password'),
        }



#:coding=utf-8:

from django import forms
from django.conf import settings
from django.conf.urls import url
from django.contrib import admin
from django.http import Http404
from django.shortcuts import redirect
from django.utils.html import escape
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.contrib.admin.helpers import AdminForm
from django.contrib.admin.utils import flatten_fieldsets
from django.forms.models import modelform_factory
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_protect
from django.shortcuts import render_to_response, get_object_or_404
from django.utils.decorators import method_decorator
from django.template import RequestContext
from django.db.transaction import atomic


csrf_protect_m = method_decorator(csrf_protect)

class UserBaseAdmin(admin.ModelAdmin):
    list_display = ('get_display_name',)
    search_fields = ('pk',)

class BasicUserCreationForm(forms.ModelForm):
    """
    A form that creates a user, with no privileges, from the given username and password.
    """
    username = forms.RegexField(label=_("Username"), max_length=30, regex=r'^[\w.@+-]+$',
        help_text = _("Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only."),
        error_messages = {'invalid': _("Enter a valid username. This value may contain only letters, numbers and @/./+/-/_ characters.")})
    password1 = forms.CharField(label=_("Password"), widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password confirmation"), widget=forms.PasswordInput,
        help_text = _("Enter the same password as above, for verification."))

    def __init__(self, *args, **kwargs):
        super(BasicUserCreationForm, self).__init__(*args, **kwargs)

    def clean_username(self):
        username = self.cleaned_data["username"]
        try:
            self._meta.model.objects.get(username=username)
        except self._meta.model.DoesNotExist:
            return username
        raise forms.ValidationError(_("A user with that username already exists."))

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1", "")
        password2 = self.cleaned_data["password2"]
        if password1 != password2:
            raise forms.ValidationError(_("The two password fields didn't match."))
        return password2

    def save(self, commit=True):
        user = super(BasicUserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user

class PasswordChangeForm(forms.Form):
    """
    A form used to change the password of a user in the admin interface.
    """
    password1 = forms.CharField(label=_("Password"), widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password (again)"), widget=forms.PasswordInput)

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super(PasswordChangeForm, self).__init__(*args, **kwargs)

    def clean_password2(self):
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return password2

    def save(self, commit=True):
        """
        Saves the new password.
        """
        self.user.set_password(self.cleaned_data["password1"])
        if commit:
            self.user.save()
        return self.user


class BasicUserAdmin(UserBaseAdmin):
    add_form_template = 'admin/newauth/basicuser/add_form.html'
    change_user_password_template = None
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('username', 'password1', 'password2')}
        ),
    )
    add_form = BasicUserCreationForm
    change_password_form = PasswordChangeForm
    search_fields = ('pk', 'username')
    ordering = ('username',)

    def get_fieldsets(self, request, obj=None):
        if not obj:
            return self.add_fieldsets
        return super(BasicUserAdmin, self).get_fieldsets(request, obj)

    def get_form(self, request, obj=None, **kwargs):
        """
        Use special form during user creation
        """
        defaults = {}
        if obj is None:
            defaults.update({
                'form': modelform_factory(self.model, self.add_form, fields='__all__'),
                'fields': flatten_fieldsets(self.add_fieldsets),
            })
        defaults.update(kwargs)
        return super(BasicUserAdmin, self).get_form(request, obj, **defaults)

    def get_urls(self):
        return [
            url(r'^(.+)/password/$', self.admin_site.admin_view(self.user_change_password))
        ] + super(BasicUserAdmin, self).get_urls()

    @csrf_protect_m
    @atomic
    def add_view(self, request, form_url='', extra_context=None):
        # It's an error for a user to have add permission but NOT change
        # permission for users. If we allowed such users to add users, they
        # could create superusers, which would mean they would essentially have
        # the permission to change users. To avoid the problem entirely, we
        # disallow users from adding users if they don't have change
        # permission.
        if not self.has_change_permission(request):
            if self.has_add_permission(request) and settings.DEBUG:
                # Raise Http404 in debug mode so that the user gets a helpful
                # error message.
                raise Http404('Your user does not have the "Change user" permission. In order to add users, Django requires that your user account have both the "Add user" and "Change user" permissions set.')
            raise PermissionDenied
        if extra_context is None:
            extra_context = {}
        defaults = {
            'auto_populated_fields': (),
            'username_help_text': self.model._meta.get_field('username').help_text,
        }
        extra_context.update(defaults)
        return super(BasicUserAdmin, self).add_view(request, form_url, extra_context)

    def user_change_password(self, request, id):
        if not self.has_change_permission(request):
            raise PermissionDenied
        user = get_object_or_404(self.model, pk=id)
        if request.method == 'POST':
            form = self.change_password_form(user, request.POST)
            if form.is_valid():
                form.save()
                messages.success(request, _('Password changed successfully.'))
                return redirect('..')
        else:
            form = self.change_password_form(user)

        fieldsets = [(None, {'fields': form.base_fields.keys()})]
        adminForm = AdminForm(form, fieldsets, {})

        return render_to_response(self.change_user_password_template or 'admin/newauth/basicuser/change_password.html', {
            'title': _('Change password: %s') % escape(str(user)),
            'adminForm': adminForm,
            'form': form,
            'is_popup': '_popup' in request.REQUEST,
            'add': True,
            'change': False,
            'has_delete_permission': False,
            'has_change_permission': True,
            'has_absolute_url': False,
            'opts': self.model._meta,
            'original': user,
            'save_as': False,
            'show_save': True,
            'root_path': self.admin_site.root_path,
        }, context_instance=RequestContext(request))



#:coding=utf-8:

import re

from django.contrib.auth import REDIRECT_FIELD_NAME
from django.shortcuts import redirect, render
from django.utils.translation import ugettext as _
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.cache import never_cache
from django.conf import settings

from newauth.api import login as auth_login, logout as auth_logout
from newauth.forms import BasicAuthForm

@csrf_protect
@never_cache
def login(request, next_page=None,
          template_name='registration/login.html',
          redirect_field_name=REDIRECT_FIELD_NAME,
          authentication_form=BasicAuthForm):
    """Displays the login form and handles the login action."""

    if request.method == "POST":
        redirect_to = request.POST.get(redirect_field_name, '')
        form = authentication_form(data=request.POST)

        if form.is_valid():
            if hasattr(request, 'session') and not request.session.test_cookie_worked():
                from django.forms.forms import NON_FIELD_ERRORS
                # Add an error to the form causing it to invalidate.
                form._errors[NON_FIELD_ERRORS] = form.error_class([_("Your Web browser doesn't appear to have cookies enabled. Cookies are required for logging in.")])
                # memo: this feature is removed on Django: https://github.com/django/django/pull/644
            else:
                # Light security check -- make sure redirect_to isn't garbage.
                if not redirect_to or ' ' in redirect_to:
                    redirect_to = next_page or settings.LOGIN_REDIRECT_URL
                
                # Heavier security check -- redirects to http://example.com should 
                # not be allowed, but things like /view/?param=http://example.com 
                # should be allowed. This regex checks if there is a '//' *before* a
                # question mark.
                elif '//' in redirect_to and re.match(r'[^\?]*//', redirect_to):
                    redirect_to = next_page or settings.LOGIN_REDIRECT_URL
               
                # Okay, security checks complete. Log the user in.
                auth_login(request, form.get_user())

                if hasattr(request, 'session') and request.session.test_cookie_worked():
                    request.session.delete_test_cookie()

                return redirect(redirect_to)

    else:
        redirect_to = request.GET.get(redirect_field_name, '')
        form = authentication_form()

    if hasattr(request, 'session'):
        request.session.set_test_cookie()
    
    return render(request, template_name, context={
        'form': form,
        redirect_field_name: redirect_to,
    })

def logout(request, next_page=None, template_name='registration/logged_out.html', redirect_field_name=REDIRECT_FIELD_NAME):
    "Logs out the user and displays 'You are logged out' message."
    auth_logout(request)
    if next_page is None:
        redirect_to = request.REQUEST.get(redirect_field_name, getattr(settings, 'LOGOUT_REDIRECT_URL', ''))
        if redirect_to:
            return redirect(redirect_to)
        else:
            return render(request, template_name, context={
                'title': _('Logged out')
            })
    else:
        # Redirect to this page until the session has been cleared.
        return redirect(next_page or request.path)

def logout_then_login(request, login_url=None):
    "Logs out the user if he is logged in. Then redirects to the log-in page."
    return logout(request, login_url or settings.LOGIN_URL)



#:coding=utf-8:

from newauth.models import (
    get_user_model, get_anonymous_user_model
)

__all__ = (
    'BaseAuthBackend',
    'ModelAuthBackend',
    'BasicUserBackend',
)

class BaseAuthBackend(object):
    """
    An base authentication backend class. Authentication
    backends should inherit from this class.
    """
    def __init__(self, backend_name):
        self.backend_name = backend_name

    def authenticate(self, **credentials):
        raise NotImplemented("You must implement the authenticate() method!")

    def get_user(self, user_id):
        raise NotImplemented("You must implement the get_user() method!")

class ModelAuthBackend(BaseAuthBackend):
    """
    An abstract authentication backend that loads
    users based on the get_user_model() method.

    Subclasses only need to implement the authenticate()
    method unless they want to override how
    get_user() works.
    """

    @property
    def user_model(self):
        return get_user_model(self.backend_name)

    @property
    def anon_user_model(self):
        return get_anonymous_user_model(self.backend_name)

    def get_user(self, user_id):
        try:
            return self.user_model.objects.get(pk=user_id)
        except self.user_model.DoesNotExist:
            return None

class BasicUserBackend(ModelAuthBackend):
    """
    A simple backend to authenticate any subclass
    of BasicUserBase.
    """
    def authenticate(self, username=None, password=None):
        try:
            user = self.user_model.objects.get(username=username)
            if user.check_password(password):
                return user
        except self.user_model.DoesNotExist:
            pass
        return None



#!/usr/bin/env python
#:coding=utf-8:

from setuptools import setup, find_packages
from newauth import VERSION


def read(filename):
    with open(filename) as f:
        return f.read()


setup (
    name='django-newauth',
    version=VERSION,
    description='Authentication for Django done right',
    long_description=read('README.rst') + read('ChangeLog.rst'),
    author='BeProud',
    author_email='project@beproud.jp',
    url='http://django-newauth.rtfd.io/',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Plugins',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'Django>=1.8',
    ],
    test_suite='tests.main',
    zip_safe=False,
)



#:coding=utf-8:

from django.conf.urls import url

from newauth.views import login, logout


urlpatterns = [
    url(r'^login/$', login, name='newauth_login'),
    url(r'^logout/$', logout, name='newauth_logout'),
]



#:coding=utf-8:

"""
Alex Gaynor will kill me
"""
import hashlib

from django.db import models
from django.utils.encoding import smart_str
from django.utils.lru_cache import lru_cache
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from newauth.constants import (
    DEFAULT_SESSION_KEY,
    DEFAULT_USER_BACKENDS,
    DEFAULT_USER_PROPERTY,
    DEFAULT_PASSWORD_ALGO,
)

__all__ = (
    'UserBase',
    'AnonymousUserBase',
    'BasicUserManager',
    'BasicUserBase',
    'BasicAnonymousUser',

    'load_backends',
    'get_backends',
    'authenticate',
    'login',
    'logout',
    'get_user_from_request',
    'get_user',
)


class UserBase(models.Model):
    """
    Base User class
    
    Primary key can be defined in sub classes. This
    class makes no assumptions about the format of the
    primary key. Only a pk property (the primary key might
    be something other than id) and the methods
    implemented below can be assumed are present.

    This class also makes no assumptions about underlying
    implementations like what fields are on the User object
    or the table name.
    """
    def is_authenticated(self):
        """
        Returns whether a user is authenticated or not.
        The default is for this method to always return
        true for subclasses of UserBase and False
        for subclasses of AnonymousUserBase
        """
        return True

    def is_anonymous(self):
        return False

    def get_display_name(self):
        """
        Name for display. Usually a username or
        something similar. Usually used for
        public pages. This method should return
        a unicode object.
        """
        from django.utils.encoding import force_unicode
        return force_unicode(self.pk)
    get_display_name.short_description = _('display name')

    def get_real_name(self):
        """
        The user's real name or closest approximation.
        Usually used for private pages etc.
        """
        return self.get_display_name()
    get_display_name.short_description = _('name')

    def __unicode__(self):
        return self.get_display_name()

    class Meta:
        abstract = True
        verbose_name = _('user')
        verbose_name_plural = _('users')


class AnonymousUserBase(object):
    """
    A simple anonymous user.
    """
    pk = None

    def __eq__(self, other):
        return isinstance(other, self.__class__)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return 1 # instances always return the same hash value

    def save(self):
        raise NotImplementedError

    def delete(self):
        raise NotImplementedError

    def is_authenticated(self):
        return False 

    def is_anonymous(self):
        return True 

    def get_display_name(self):
        return u'Guest'
    
    def get_real_name(self):
        return self.get_display_name()

    def __unicode__(self):
        return self.get_display_name()


UNUSABLE_PASSWORD = '!' # This will never be a valid hash

# md5, sha1, crypt
PASSWORD_ALGO = getattr(settings, 'NEWAUTH_PASSWORD_ALGO', DEFAULT_PASSWORD_ALGO)


def get_hexdigest(algorithm, salt, raw_password):
    """
    Returns a string of the hexdigest of the given plaintext password and salt
    using the given algorithm ('md5', 'sha1' or 'crypt').
    """
    raw_password, salt = smart_str(raw_password), smart_str(salt)
    if algorithm == 'crypt':
        try:
            import crypt
        except ImportError:
            raise ValueError('"crypt" password algorithm not supported in this environment')
        return crypt.crypt(raw_password, salt)

    try:
        hash_method = getattr(hashlib, algorithm)
        return hash_method(salt+raw_password).hexdigest()
    except AttributeError:
        raise ValueError("Got unknown password algorithm type in password.")


def check_password(raw_password, enc_password):
    """
    Returns a boolean of whether the raw_password was correct. Handles
    encryption formats behind the scenes.
    """
    algo, salt, hsh = enc_password.split('$')
    return hsh == get_hexdigest(algo, salt, raw_password)


class BasicUserManager(models.Manager):
    """
    A default manager that can be used with models that
    subclass BasicUserBase.
    """
    def create_user(self, username, password=None): 
        user = self.model(username=username)
        user.set_password(password)
        user.save(using=self._db)
        return user

class BasicUserBase(UserBase):
    """
    A basic user that is authenticated with a
    username and password.

    This class can be subclassed when using
    a simple username password auth system.
    """
    username = models.CharField(_('username'), max_length=30, unique=True, help_text=_("Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only."))
    password = models.CharField(_('password'), max_length=128)

    objects = BasicUserManager()

    def set_password(self, raw_password):
        """
        Sets the password of the user. Alorithm
        """
        if raw_password is None:
            self.set_unusable_password()
        else:
            import random
            algo = PASSWORD_ALGO
            salt = get_hexdigest(algo, str(random.random()), str(random.random()))[:5]
            hsh = get_hexdigest(algo, salt, raw_password)
            self.password = '%s$%s$%s' % (algo, salt, hsh)

    def check_password(self, raw_password):
        """
        Returns a boolean of whether the raw_password was correct. Handles
        encryption formats behind the scenes.
        """
        return check_password(raw_password, self.password)

    def set_unusable_password(self):
        # Sets a value that will never be a valid hash
        self.password = UNUSABLE_PASSWORD

    def has_usable_password(self):
        return self.password != UNUSABLE_PASSWORD

    def get_display_name(self):
        return self.username
    get_display_name.short_description = _('username')

    class Meta:
        abstract = True

class BasicAnonymousUser(AnonymousUserBase):
    username=''
    password=None
    def set_password(self, raw_password):
        raise NotImplementedError

    def check_password(self, raw_password):
        raise NotImplementedError

    def has_usable_password(self, raw_password):
        raise NotImplementedError

def _get_backend_data():
    from django.conf import settings

    backend_data = getattr(settings, 'NEWAUTH_BACKENDS', DEFAULT_USER_BACKENDS)
    for key, given_data in backend_data.iteritems():
        data = DEFAULT_USER_BACKENDS['default'].copy() 
        if isinstance(given_data['backend'], basestring):
            given_data['backend'] = (given_data['backend'],)
        data.update(given_data)
        backend_data[key] = data
    return backend_data

def import_string(import_name, silent=False):
    """Imports an object based on a string.  This is useful if you want to
    use import paths as endpoints or something similar.  An import path can
    be specified either in dotted notation (``xml.sax.saxutils.escape``)
    or with a colon as object delimiter (``xml.sax.saxutils:escape``).

    If `silent` is True the return value will be `None` if the import fails.

    :param import_name: the dotted name for the object to import.
    :param silent: if set to `True` import errors are ignored and
                   `None` is returned instead.
    :return: imported object
    """
    # force the import name to automatically convert to strings
    if isinstance(import_name, unicode):
        import_name = str(import_name)
    try:
        if ':' in import_name:
            module, obj = import_name.split(':', 1)
        elif '.' in import_name:
            module, obj = import_name.rsplit('.', 1)
        else:
            return __import__(import_name)
        # __import__ is not able to handle unicode strings in the fromlist
        # if the module is a package
        if isinstance(obj, unicode):
            obj = obj.encode('utf-8')
        return getattr(__import__(module, None, None, [obj]), obj)
    except (ImportError, AttributeError):
        if not silent:
            raise


@lru_cache()
def load_backends(backend_name):
    """
    Load the auth backend with the given name
    """
    backend_data = _get_backend_data().get(backend_name)
    if not backend_data:
        raise ImproperlyConfigured('The specified backend "%s" does not exist. Is NEWAUTH_BACKENDS a correctly defined dict?' % backend_name)

    backends = []
    for path in backend_data['backend']:
        try:
            cls = import_string(path)
        except (ImportError, AttributeError), e:
            raise ImproperlyConfigured('Error importing authentication backend %s: "%s"' % (path, e))
        except ValueError, e:
            raise ImproperlyConfigured('Error importing authentication backends. Is NEWAUTH_BACKENDS a correctly defined dict?')
        backends.append(cls(backend_name))
    return backends


def get_backends(backend_names=None):
    """
    Load all auth backends and return them as a (name, backend) two tuple.
    """
    all_backend_names = _get_backend_data().keys()
    if backend_names is None:
        backend_names = all_backend_names
    else:
        backend_names = filter(lambda b: b in backend_names, all_backend_names)

    backends = []
    for backend_name in backend_names:
        backends.append((backend_name, load_backends(backend_name)))
    return backends

# _backend_name has an underscore so it won't conflict with any
# possible credential keyword arguments
def authenticate(_backend_name=None, **credentials):
    """
    Authenticate a user with using the available auth
    backends. If a _backend_name is provided require
    authentication via the backend with that name.

    If the given credentials are valid, return a User object
    as provided by the backend.
    """
    if _backend_name:
        backends = [(_backend_name, load_backends(_backend_name))]
    else:
        backends = get_backends()

    for backend_name, backend_list in backends:
        for backend in backend_list:
            try:
                user = backend.authenticate(**credentials)
            except TypeError:
                # This backend doesn't accept these credentials as arguments. Try the next one.
                continue
            if user is None:
                continue
            user._backend = backend
            user._backend_name = backend_name
            # TODO: login succeeded. log as info.
            # TODO: authenticate signal
            return user
    # TODO: user failed login. log this as a warning
    # login failures could indicate suspicious activity.
    return None

def login(request, user, backend_name=None):
    """
    Log the user in given the request object. This will
    save any data needed to auth the user (user id and backend name)
    using the auth session storage (not necessarily django sessions).
    """
    from django.conf import settings

    if not hasattr(user, '_backend_name') and backend_name is None:
        # User has no backend set. Cannot login.
        raise ValueError("You must retrieve the user object via a backend "
                         "or via the get_user() function or specify"
                         "a backend_name to the login() function.")

    if not hasattr(request, 'session'):
        raise ImproperlyConfigured("You must add "
                                   "django.contrib.sessions.SessionMiddleware "
                                   "to your MIDDLEWARE_CLASSES in order to support login.")

    # TODO: pre login signal

    session_key = getattr(settings, 'NEWAUTH_SESSION_KEY', DEFAULT_SESSION_KEY)
    current_uid = request.session.get(session_key, {}).get('uid')
    if current_uid and user.pk != current_uid:
        # To avoid reusing another user's session, create a new, empty
        # session if the existing session corresponds to a different
        # authenticated user.
        # TODO: log as info
        request.session.flush()
    else:
        # TODO: log as info
        request.session.cycle_key()

    user_data = {
        'uid': user.pk,
        'bn': backend_name or user._backend_name,
    }
    request.session[session_key] = user_data

    user_prop = getattr(settings, '_USER_PROPERTY', DEFAULT_USER_PROPERTY)
    if hasattr(request, user_prop): 
        setattr(request, user_prop, user)

    # TODO: post login signal

def logout(request):
    """
    Logs the user out. This will clear the user's login session data
    from the auth session storage.
    """
    from django.conf import settings
    # TODO: pre logout signal

    user_prop = getattr(settings, 'NEWAUTH_USER_PROPERTY', DEFAULT_USER_PROPERTY)
    if hasattr(request, user_prop):
        from newauth.models import (
            AnonymousUser, get_anonymous_user_model,
        )

        user = getattr(request, user_prop)
        if hasattr(user, '_backend_name'):
            # Set the current request user to the anonymous user
            # given by the backend of the user being logged out.
            setattr(request, user_prop, get_anonymous_user_model(user._backend_name)())
        elif not isinstance(user, AnonymousUserBase):
            # if the user has no backend set and is not already
            # an anonymous user then set the user to the default
            # anonymous user.
            setattr(request, user_prop, AnonymousUser())
    request.session.flush()

    # TODO: post logout signal

def get_user_from_request(request):
    """
    Gets a user from a request. Used by
    the AuthMiddleware.
    """
    from newauth.models import AnonymousUser
    from django.conf import settings

    if not hasattr(request, 'session'):
        return AnonymousUser()

    session_key = getattr(settings, 'NEWAUTH_SESSION_KEY', DEFAULT_SESSION_KEY)
    session_data = request.session.get(session_key, {})
    user_id = session_data.get("uid")
    backend_name = session_data.get('bn')

    try:
        return get_user(user_id, backend_name)
    except ImproperlyConfigured:
        # We don't trust the backend name given by the request here.
        # Even though storage should be secure if the backend configuration
        # changes this could cause errors to occur so we suppress them here.
        # TODO: log this as a warning
        return AnonymousUser()

def get_user(user_id, backend_name=None):
    """
    Get the user from a user_id. If a backend_name
    is given then the backend with that name will
    be used.

    If a backend_name is not given (default) then
    all backends are tried.

:
    This method should be used to get a user
    rather than getting the user directly from
    the database as it associates the correct
    backend with the user.
    """
    # Avoid making DB queries when we know that the user
    # doesn't exist.
    if user_id is not None:
        if backend_name:
            backends = [(backend_name, load_backends(backend_name))]
        else:
            backends = get_backends()

        for name, backend_list in backends:
            for backend in backend_list:
                user = backend.get_user(user_id=user_id)
                if user:
                    user._backend = backend
                    user._backend_name = name
                    return user

    # If the backend name is given then use
    # that backend's anonymous user
    if backend_name:
        from newauth.models import get_anonymous_user_model
        return get_anonymous_user_model(backend_name)()

    # If no backend is specified and no
    # backend can get the user then use
    # the default anonymous user class.
    from newauth.models import AnonymousUser
    return AnonymousUser()


